/* Wingate Monitor Bot -- (c) 2001 Darcy Grexton
   Based on wgmon, (c) 1999-2001 Lucas Madar & DALnet
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <stdarg.h>
#include "h.h"
#include "settings.h"
#include "conf.h"

#define cop(x) if(strcasecmp(opt, x) == 0)

extern char CONFFILE[256];
char SERVIP[16], USER[128], NICK[32], NICKPASS[64], ONICK[32], OPASS[32], SCIP[16];
char MYCHAN[64], INVCHAN[64];
int SERVPORT, SCPORT;
unsigned long BINDIP;

/* Init all the configuration variables. */
void initconfig ()
{
    SERVPORT = SCPORT = BINDIP = 0;
    SERVIP[0] = USER[0] = NICK[0] = NICKPASS[0] = ONICK[0] = OPASS[0] = SCIP[0] = '\0';
    MYCHAN[0] = INVCHAN[0] = '\0';
}

/* Report a wingate ? */
int olp (int isonline, char *pattern, ...)
{
    va_list va;
    char buf[2048];

    va_start (va, pattern);
    vsprintf (buf, pattern, va);
    va_end (va);

    if (!isonline)
	printf (buf);
    else
	ssend ("PRIVMSG %s :%s", MYCHAN, buf);

    return 0;
}

/* Load the conf */
int loadconf (int isonline)
{
    FILE *cf;
    char *opt, *val, *tmp, lbuf[512];
    int retval = CONF_SUCCESS;

    cf = fopen (CONFFILE, "r");

    if (!cf)
    {
	olp (isonline, "Error! Could not load %s!\n", CONFFILE);
	return 0;
    }

    while (fgets (lbuf, 512, cf))
    {
	while ((tmp = strchr (lbuf, '\r')))
	    *tmp = '\0';
	while ((tmp = strchr(lbuf, '\n')))
	    *tmp = '\0';

	opt = tmp = lbuf;

	if (!*opt || *opt == '#' || *opt == ';')
	    continue;

	while (*tmp && *tmp != ' ')
	    tmp++;

	*tmp = '\0';
	tmp++;

	while (*tmp && *tmp == ' ')
	    tmp++;

	val = tmp;

	while (*tmp && *tmp != ' ')
	    tmp++;

	*tmp = '\0';

	if (!*val)
	    continue;

	cop ("Server")
	{
	    struct in_addr ia;
	    struct hostent *he;

	    if ((ia.s_addr = inet_addr(val)) == -1)
	    {
		he = gethostbyname(val);

		if (!he)
		{
		    if (!isonline)

		    printf ("Cannot resolve %s\n", val);
		    continue;
		}

		memcpy ((char *)&ia.s_addr, he->h_addr_list[0],
		    sizeof(unsigned long));
	    }

	    strcpy (SERVIP, inet_ntoa(ia));

	    if (!isonline)
		printf ("Server IP: %s\n", SERVIP);

	    continue;
	}

	cop ("Port")
	{
	    SERVPORT = atol (val);

	    if (!isonline)
		printf ("Port: %d\n", SERVPORT);

	    continue;
	}

	cop ("User")
	{
	    if (strlen (val) > 10)
	    {
		olp (isonline, "USER info too long.");
		continue;
	    }

	    sprintf (USER, "%s my.host server.host :Wingate/SOCKS detection bot", val);

	    continue;
	}

	cop ("Nick")
	{
	    char onick[32];

	    if (strlen(val) > 30)
	    {
		olp (isonline, "NICK too long.");
		continue;
	    }

	    strcpy (onick, NICK);
	    strcpy (NICK, val);

	    if (strcmp (onick, NICK))
		retval |= CONF_NICKCHG;

	    continue;
	}

	cop ("NickPass")
	{
	    if (strlen(val) > 60)
	    {
		olp (isonline, "Nick PASS too long.");
		continue;
	    }

	    strcpy (NICKPASS, val);
	    continue;
	}

	cop ("OperNick")
	{
	    if (strlen (val) > 30)
	    {
		olp (isonline, "Oper NICK too long.");
		continue;
	    }

	    strcpy (ONICK, val);
	    continue;
	}

	cop ("OperPass")
	{
	    if (strlen (val) > 30)
	    {
		olp (isonline, "Oper PASS too long.");
		continue;
	    }

	    strcpy (OPASS, val);
	    continue;
	}

	cop ("DestServ")
	{
	    struct in_addr ia;
	    struct hostent *he;

	    char oldscip[32];

	    if ((ia.s_addr = inet_addr (val)) == -1)
	    {
		he = gethostbyname (val);

		if (!he)
		{
		    olp(isonline, "Cannot resolve dest server %s\n", val);
		    continue;
		}
		
		memcpy ((char *)&ia.s_addr, he->h_addr_list[0],
		sizeof (unsigned long));
	    }

	    if (ia.s_addr == inet_addr ("127.0.0.1"))
	    {
		olp (isonline,
		    "Error: destserv must be the actual IP of your server, not  localhost!\n");
		continue;
	    }

	    strcpy (oldscip, SCIP);

	    strcpy (SCIP, inet_ntoa(ia));

	    if (strcmp (oldscip, SCIP))
	    {
		retval |= CONF_UPDATES4;
		olp (isonline, "Socks dest IP: %s\n", SCIP);
	    }

	    continue;
	}

	cop ("DestPort")
	{
	    int oldp;

	    oldp = SCPORT;
	    SCPORT = atol (val);

	    if (oldp != SCPORT)
	    {
		retval |= CONF_UPDATES4;
		olp (isonline, "Socks dest port: %d\n", SERVPORT);
	    }

	    continue;
	}

	cop ("Report")
	{
	    char oldc[64];

	    if (strlen (val) > 62)
	    {
		olp (isonline, "Report chan too long.");
		continue;
	    }

	    strcpy (oldc, MYCHAN);

	    if (strcmp (oldc, val))
	    {
		retval |= CONF_CHANCHG;
		olp (isonline, "Report channel: %s\n", val);
	    }

	    strcpy (MYCHAN, val);

	    continue;
	}

	cop ("Invite")
	{
	    if (strlen (val) > 62)
	    {
		olp (isonline, "Invite chan too long.");
		continue;
	    }

	    strcpy (INVCHAN, val);

	    continue;
	}

	cop ("BindIP")
	{
	    struct in_addr ia;
	    struct hostent *he;

	    unsigned long oldbindip;

	    if ((ia.s_addr = inet_addr(val)) == -1)
	    {
		he = gethostbyname (val);

		if (!he)
		{
		    olp (isonline, "Cannot resolve bind ip host %s\n", val);
		    continue;
		}

		memcpy ((char *)&ia.s_addr, he->h_addr_list[0],
		    sizeof(unsigned long));
	    }

	    if (ia.s_addr == inet_addr("127.0.0.1"))
	    {
		olp (isonline, "Error: bindip cannot be localhost!\n");
		continue;
	    }

	    oldbindip = BINDIP;
	    BINDIP = ia.s_addr;

	    if (BINDIP != oldbindip)
	    {
		if (isonline)
		{
		    olp (isonline,
			"BindIP will not take effect until a restart.\n");
		}
		else
		    olp (isonline, "Binding ip: %s\n", inet_ntoa (ia));
	    }

	    continue;
	}

	if (!isonline)
	    printf ("Unknown option: %s\n", opt);
    }

    fclose(cf);

    if (!isonline)
    {
	if (SERVPORT == 0)
	{
	    retval = 0;
	    printf ("No server port specified.\n");
	}

	if (SERVIP[0] == '\0')
	{
	    retval = 0;
	    printf ("No server address specified.\n");
	}

	if (USER[0] == '\0')
	{
	    retval = 0;
	    printf ("No USER specified.\n");
	}

	if (NICK[0] == '\0')
	{
	    retval = 0;
	    printf ("No nickname specified.\n");
	}

	if (ONICK[0] == '\0')
	{
	    retval = 0;
	    printf ("No oper nickname specified.\n");
	}

	if (OPASS[0] == '\0')
	{
	    retval = 0;
	    printf ("No oper password specified.\n");
	}

	if (SCIP[0] == '\0')
	{
	    retval = 0;
	    printf ("No destination server specified.\n");
	}

	if (MYCHAN[0] == '\0')
	{
	    retval = 0;
	    printf ("No report channel specified.\n");
	}

	if (SCPORT == 0)
	{
	    retval = 0;
	    printf ("No socks destination port specified.\n");
	}
    }

    if (!isonline)
	printf("\n");

    return retval;
}
